/*
 * redspecial.c: generazione della scena per la Red Special tramite la libreria
 *               descriptor
 */

#include "descriptor/descriptor.h"
#include <math.h>


void costruisci_scala(float *risultato);

ITEM posiziona_segnatasti(float *scala);

ITEM costruisci_ponte(void);

ITEM posiziona_corde(float meccaniche_x[], float meccaniche_y[]);

ITEM posiziona_meccaniche(float out_x[], float out_y[]);

ITEM costruisci_redspecial(void);

ITEM costruisci_porta_chitarra(void);

ITEM costruisci_stanza(void);


ITEM bianco;

int main(void)
{
        ITEM scena, redspecial, porta_chitarra, stanza;

        /* luci */
        set_background(0.85, 0.85, 0.85, TRUE);
        set_ambient_light(0.8, 0.8, 0.8, 0.7);
        create_warn_light(22.37, 11.43, 45.69, -0.466, -0.242, -0.850,
                          1, 1, 1, 1.6, 0, 1, 36, 0, 0, 0, 0, 0, "luce1");
        create_warn_light(7.666, -43.549, 39.918, -0.136, 0.743, -0.655,
                          1, 1, 1, 1.6, 0, 2, 36, 0, 0, 0, 0, 0, "luce2");

        /* attributi */
        bianco = create_attribute("bianco");
        set_color(bianco, 1, 1, 1, 0, 1);

        redspecial = costruisci_redspecial();
        porta_chitarra = costruisci_porta_chitarra();

        set_z_rotate(redspecial, 90, 1);
        set_x_rotate(redspecial, 90, 1);
        set_scale(porta_chitarra, 0.8, 0.8, 0.8);
        set_translate(porta_chitarra, 0, 0, -0.2);
        set_translate(redspecial, 0, -1, 0);
        set_x_rotate(redspecial, -4, 1);

        stanza = costruisci_stanza();
        set_z_rotate(stanza, -20, 1);

        scena = create_list(redspecial, "scena");
        scena = insert_in_list(scena, porta_chitarra);
        scena = insert_in_list(scena, stanza);

        save_scene("redspecial.md", "scena", scena);

        return 0;
}


ITEM costruisci_redspecial(void)
{
        int i, j;

        /* curve per la chitarra */
        ITEM cassa, bordo_sup, bordo_inf,
                manico_sup, manico_inf, paletta,
                paletta_inf, tastiera_testa, tastiera_coda, tasto, tasto_24,
                bp_esterno, bp_interno, battipenna_ponte;
        ITEM pick_ponte, pick_mezzo, pick_manico;
        ITEM controllo_vol, controllo_tono, capotasto;
        ITEM attacco_tracolla_sopra, attacco_tracolla_sotto;
        ITEM s[3][2];

        float delta, delta2, delta3;
        char buffer[50];
        float scala[25];

        /* attributi */
        ITEM plastica_nera, metallo_corde;
        ITEM cromato, avorio;

        /* liste di oggetti composti */
        ITEM redspecial, corpo, manico, bordi, pickup, tastiera, battipenna;
        ITEM ponte, controlli, interruttori, tasti, tappo_leva_tremolo;
        ITEM meccaniche;
        ITEM segnatasti;
        ITEM corde;

        float meccaniche_x[6];
        float meccaniche_y[6];

        plastica_nera = create_attribute("plastica_nera");
        set_color(plastica_nera, 0, 0, 0, 0.2, 1);
        set_reflectivity(plastica_nera, 0.04, 2);

        metallo_corde = create_attribute("metallo_corde");
        set_color(metallo_corde, 0.4, 0.4, 0.4, 0, 1);

        cromato = create_attribute("cromato");
        set_color(cromato, 0.33, 0.33, 0.33, 0.3, 0.3);
        set_reflectivity(cromato, 0.39, 2);

        avorio = create_attribute("avorio");
        set_color(avorio, 1, 1, 0.94, 0, 1);

        /* parti della chitarra */
        cassa = read_nurbs("corpo_cassa.db");
        bordo_sup = read_nurbs("bordo_cassa_sopra.db");
        bordo_inf = read_nurbs("bordo_cassa_sotto.db");
        manico_sup = read_nurbs("manico_sopra.db");
        manico_inf = read_nurbs("manico_sotto.db");
        paletta = read_nurbs("paletta.db");
        paletta_inf = read_nurbs("paletta_sotto.db");
        tastiera_testa = read_nurbs("tastiera.db");
        tastiera_coda = read_nurbs("coda.db");
        capotasto = read_nurbs("capotasto.db");
        bp_interno = read_nurbs("interno.db");
        battipenna_ponte = read_nurbs("battipenna_ponte.db");
        pick_manico = read_nurbs("pickup.db");
        pick_mezzo = create_copy(pick_manico, "pickup_mezzo");
        pick_ponte = create_copy(pick_manico, "pickup_ponte");
        attacco_tracolla_sopra = read_nurbs("attacco_tracolla_sopra.db");
        attacco_tracolla_sotto = read_nurbs("attacco_tracolla_sotto.db");

        controllo_vol = read_nurbs("controllo_volume.db");
        controllo_tono = create_copy(controllo_vol, "controllo_tono");

        s[0][0] = read_nurbs("switch.db");
        delta = 0.55;
        delta2 = 0.1;
        for (i = 0; i < 3; ++i) {
                for (j = 0; j < 2; ++j) {
                        if (i || j) {
                                snprintf(buffer, 50, "switch_%d_%d", i, j);
                                s[i][j] = create_copy(s[0][0], buffer);
                                set_translate(s[i][j], delta * i,
                                              delta * j + delta2 * i, 0);
                                interruttori = insert_in_list(interruttori,
                                                              s[i][j]);
                        } else {
                                interruttori = create_list(s[0][0],
                                                           "interruttori");
                        }
                }
        }
        set_attribute(interruttori, bianco);

        set_translate(pick_mezzo, -1, 0, 0);
        set_translate(pick_ponte, -2, 0, 0);
        set_translate(controllo_tono, 0.53, -0.42, 0);

        meccaniche = posiziona_meccaniche(meccaniche_x, meccaniche_y);

        corde = posiziona_corde(meccaniche_x, meccaniche_y);

        /* --- posiziono i tasti sulla tastiera... --- */
        tasto_24 = read_nurbs("tasto_24.db");
        costruisci_scala(scala);
        delta = 0.324; /* lunghezza "base" di meta` tasto */
        delta2 = 0.4;  /* lunghezza di meta` dell'ultimo tasto */
        delta3 = 0.076 / 8; /* tangente dell'angolatura della tastiera
                             * (end-start) */
        tasti = create_list(tasto_24, "tasti");
        for (i = 1; i < 24; ++i) {
                float s = (scala[i] * delta3 + delta) / delta2;
                snprintf(buffer, 50, "tasto_%d", i);
                tasto = create_copy(tasto_24, buffer);
                set_translate(tasto, 8.0 - scala[i], 0, 0);
                set_scale(tasto, 1, s, 1);
                tasti = insert_in_list(tasti, tasto);
        }
        /* --- fine tasti sulla tastiera --- */

        /* segnatasti */
        segnatasti = posiziona_segnatasti(scala);

        manico = create_list(manico_sup, "manico");
        manico = insert_in_list(manico, manico_inf);
        manico = insert_in_list(manico, paletta);
        manico = insert_in_list(manico, paletta_inf);

        bordi = create_list(bordo_sup, "bordi");
        bordi = insert_in_list(bordi, bordo_inf);

        corpo = create_list(cassa, "corpo");

        tastiera = create_list(tastiera_testa, "tastiera");
        tastiera = insert_in_list(tastiera, tastiera_coda);
        tastiera = insert_in_list(tastiera, capotasto);
        tastiera = insert_in_list(tastiera, tasti);
        tastiera = insert_in_list(tastiera, segnatasti);

        battipenna = create_list(bp_interno, "battipenna");
        battipenna = insert_in_list(battipenna, battipenna_ponte);

        pickup = create_list(pick_manico, "pickup");
        pickup = insert_in_list(pickup, pick_mezzo);
        pickup = insert_in_list(pickup, pick_ponte);

        ponte = costruisci_ponte();
        tappo_leva_tremolo = read_nurbs("leva_tremolo_tappo.db");

        controlli = create_list(controllo_vol, "controlli");
        controlli = insert_in_list(controlli, controllo_tono);

        set_attribute(bordi, bianco);
        set_attribute(battipenna, plastica_nera);
        set_attribute(ponte, cromato);
        set_attribute(controlli, cromato);
        set_attribute(capotasto, avorio);
        set_attribute(meccaniche, cromato);
        set_attribute(tasti, cromato);
        set_attribute(pickup, cromato);
        set_attribute(segnatasti, bianco);
        set_attribute(tappo_leva_tremolo, bianco);
        set_attribute(corde, metallo_corde);
        set_attribute(attacco_tracolla_sopra, cromato);
        set_attribute(attacco_tracolla_sotto, cromato);

        // texture mapping per la tastiera...
        set_projection_texture(tastiera_testa, bianco,"textures/ebony.hr",
                               0, 1, 0.3, 0.3);
        set_projection_texture(tastiera_coda, bianco,"textures/ebony.hr",
                               0, 1, 0.3, 0.3);
        // texture mapping per il corpo...
        set_z_rotate(corpo, 90, 1);
        set_projection_texture(corpo, bianco,"textures/mahogany.hr",
                               2, 1, 0.3, 0.3);
        set_z_rotate(corpo, -90, 1);

        set_projection_texture(manico, bianco,"textures/mahogany.hr",
                               0, 1, 0.3, 0.3);

        redspecial = create_list(corpo, "redspecial");
        redspecial = insert_in_list(redspecial, manico);
        redspecial = insert_in_list(redspecial, bordi);
        redspecial = insert_in_list(redspecial, tastiera);
        redspecial = insert_in_list(redspecial, battipenna);
        redspecial = insert_in_list(redspecial, pickup);
        redspecial = insert_in_list(redspecial, ponte);
        redspecial = insert_in_list(redspecial, controlli);
        redspecial = insert_in_list(redspecial, interruttori);
        redspecial = insert_in_list(redspecial, meccaniche);
        redspecial = insert_in_list(redspecial, tappo_leva_tremolo);
        redspecial = insert_in_list(redspecial, corde);
        redspecial = insert_in_list(redspecial, attacco_tracolla_sopra);
        redspecial = insert_in_list(redspecial, attacco_tracolla_sotto);

        return redspecial;
}


void costruisci_scala(float *risultato)
{
        const float lunghezza = 8.0 * 4.0 / 3.0;
        const float c = 17.817;
        int i;

        risultato[0] = 0;

        for (i = 1; i < 25; ++i) {
                risultato[i] = (lunghezza - risultato[i-1])/c + risultato[i-1];
        }
}


ITEM posiziona_segnatasti(float *scala)
{
        static int posizioni_segnatasti[] = {
                3, 5, 7, 9, 12, 15, 17, 19, 21, 24
        };
        static int numero_di_segnatasti[] = {
                1, 1, 2, 1,  3,  1,  1,  2,  1,  3
        };
        static float y_2[] = { -0.2, 0.2 };
        static float y_3[] = { -0.2, 0, 0.2 };

        ITEM segnatasti, s1, s2;
        float x, y;
        int i, j, first;
        char buffer[50];

        first = 1;
        s1 = read_nurbs("segnatasto.db");

        for (i = 0; i < 10; ++i) {
                x = 8.0 - (scala[posizioni_segnatasti[i]-1] -
                           (scala[posizioni_segnatasti[i]-1] -
                            scala[posizioni_segnatasti[i]]) / 2);
                switch (numero_di_segnatasti[i]) {
                case 1:
                        snprintf(buffer, 50, "segnatasto_%d",
                                 posizioni_segnatasti[i]);
                        s2 = create_copy(s1, buffer);
                        set_translate(s2, x, 0, 0);
                        if (first) {
                                segnatasti = create_list(s2, "segnatasti");
                                first = 0;
                        } else {
                                segnatasti = insert_in_list(segnatasti, s2);
                        }
                        break;
                case 2:
                        for (j = 0; j < 2; ++j) {
                                snprintf(buffer, 50, "segnatasto_%d_%d",
                                         posizioni_segnatasti[i], j+1);
                                s2 = create_copy(s1, buffer);
                                set_translate(s2, x, y_2[j], 0);
                                if (first) {
                                        segnatasti = create_list(
                                                s2, "segnatasti");
                                        first = 0;
                                } else {
                                        segnatasti = insert_in_list(
                                                segnatasti, s2);
                                }
                        }
                        break;
                case 3:
                        for (j = 0; j < 3; ++j) {
                                snprintf(buffer, 50, "segnatasto_%d_%d",
                                         posizioni_segnatasti[i], j+1);
                                s2 = create_copy(s1, buffer);
                                set_translate(s2, x, y_3[j], 0);
                                if (first) {
                                        segnatasti = create_list(
                                                s2, "segnatasti");
                                        first = 0;
                                } else {
                                        segnatasti = insert_in_list(
                                                segnatasti, s2);
                                }
                        }
                        break;
                }
        }

        return segnatasti;
}


ITEM costruisci_ponte(void)
{
        ITEM ponte_davanti, ponte_mezzo, ponte_dietro, rotella, r2,
                tremolo_sopra, tremolo_sotto, leva_tremolo;
        ITEM ponte;
        int i;
        char buffer[50];
        float delta = -0.14, delta2 = 0.005;

        ponte_davanti = read_nurbs("ponte_davanti.db");
        ponte_mezzo = read_nurbs("ponte_mezzo.db");
        ponte_dietro = read_nurbs("ponte_dietro.db");
        tremolo_sopra = read_nurbs("tremolo_sopra.db");
        tremolo_sotto = read_nurbs("tremolo_sotto.db");
        leva_tremolo = read_nurbs("leva_tremolo.db");

        ponte = create_list(ponte_davanti, "ponte");
        ponte = insert_in_list(ponte, ponte_mezzo);
        ponte = insert_in_list(ponte, ponte_dietro);
        ponte = insert_in_list(ponte, tremolo_sopra);
        ponte = insert_in_list(ponte, tremolo_sotto);
        ponte = insert_in_list(ponte, leva_tremolo);

        rotella = read_nurbs("rotella.db");
        ponte = insert_in_list(ponte, rotella);

        for (i = 1; i < 6; ++i) {
                snprintf(buffer, 50, "rotella_%d", i);
                r2 = create_copy(rotella, buffer);
                set_translate(r2, i * delta2, i * delta, 0);
                ponte = insert_in_list(ponte, r2);
        }

        return ponte;
}


ITEM posiziona_corde(float meccaniche_x[], float meccaniche_y[])
{
        ITEM parti_corde[6];
        ITEM corde, tmp;

        char buffer[50];
        float delta, delta2, delta3, c, d;
        float start1 = -0.35, start2 = -0.284;
        float scale_step = 0.8;
        int i;

        tmp = read_nurbs("corda_fine.db");
        delta = 0.14;

        for (i = 0; i < 6; ++i) {
                snprintf(buffer, 50, "corda_fine_%d", i);
                parti_corde[i] = create_copy(tmp, buffer);
                if (i) {
                        set_scale(parti_corde[i], 1, pow(scale_step, i),
                                  pow(scale_step, i));
                }
                set_y_rotate(parti_corde[i], -4.8, 1);
                set_translate(parti_corde[i], -4.24, -start1 + -delta * i,
                              0.295);
                if (i) {
                        corde = insert_in_list(corde, parti_corde[i]);
                } else {
                        corde = create_list(parti_corde[i], "corde");
                }
        }

        delta2 = 0.1136;
        delta3 = 11.0;

        tmp = read_nurbs("corda_mezzo.db");

        for (i = 0; i < 6; ++i) {
                snprintf(buffer, 50, "corda_mezzo_%d", i);
                parti_corde[i] = create_copy(tmp, buffer);
                if (i) {
                        set_scale(parti_corde[i], 1, pow(scale_step, i),
                                  pow(scale_step, i));
                }
                set_y_rotate(parti_corde[i], 0.5, 1);
                c = fabs(start1 + delta * i) - fabs(start2 + delta2 * i);
                if (c > 0.001) {
                        set_z_rotate(parti_corde[i],
                                     (i < 3 ? -1 : 1) * atan(c / delta3), 0);
                }
                set_translate(parti_corde[i], -2.965, -start1 + -delta * i,
                              0.4);
                corde = insert_in_list(corde, parti_corde[i]);
        }

        tmp = read_nurbs("corda_inizio.db");

        for (i = 0; i < 6; ++i) {
                snprintf(buffer, 50, "corda_inizio_%d", i);
                parti_corde[i] = create_copy(tmp, buffer);
                c = fabs(fabs(start2 + delta2 * i) - fabs(meccaniche_y[i]));
                if (i) {
                        set_scale(parti_corde[i], 1, pow(scale_step, i),
                                  pow(scale_step, i));
                }
                set_y_rotate(parti_corde[i], 2, 1);
                d = fabs(meccaniche_x[i] - 8);
                d = sqrt(d * d + c * c);
                // 4.0 e` la lunghezza di "corda_inizio.db", d e` la lunghezza
                // del pezzo che mi serve per arrivare alla meccanica, quindi
                // scalo opportunamente...
                set_scale(parti_corde[i], d / 4.0, 1, 1);
                // adesso ruoto per raggiungere la meccanica
                if (c > 0.001) {
                        d = fabs(meccaniche_x[i] - 8.0);
                        set_z_rotate(parti_corde[i],
                                     (i < 3 ? 1 : -1) * atan(c / d), 0);
                }
                // e infine traslo avanti di 8 (perche' "corda_inizio.db" e`
                // posizionata inizialmente all'origine) e lateralmente di
                // quanto mi serve...
                set_translate(parti_corde[i], 8.02, -start2 - delta2 * i,
                              0.315);
                corde = insert_in_list(corde, parti_corde[i]);
        }

        return corde;
}


ITEM posiziona_meccaniche(float out_x[], float out_y[])
{
        ITEM meccaniche_sx[3], meccaniche_dx[3];
        /* attacchi per le corde alla paletta */
        ITEM attacco_corda;
        ITEM attacco_sotto;
        ITEM attacco_vite;
        ITEM vite;

        ITEM meccaniche;

        float delta, delta2, delta3;
        int i;
        char buffer[50];

        /* a sinistra... */
        delta = 0.5;
        delta2 = 0.5 * tan(-16.0 * M_PI / 180.0);
        delta3 = 0.5 * sin(4.0 * M_PI / 180.0);

        attacco_corda = read_nurbs("attacco_corda_sx.db");
        attacco_sotto = read_nurbs("attacco_corda_sotto_sx.db");
        attacco_vite = read_nurbs("attacco_vite_sx.db");
        vite = read_nurbs("vite_sx.db");

        meccaniche_sx[0] = create_list(attacco_corda, "meccanica_sx_0");
        meccaniche_sx[0] = insert_in_list(meccaniche_sx[0], attacco_sotto);
        meccaniche_sx[0] = insert_in_list(meccaniche_sx[0], attacco_vite);
        meccaniche_sx[0] = insert_in_list(meccaniche_sx[0], vite);

        out_x[0] = 8.592;
        out_y[0] = 0.4;

        for (i = 1; i < 3; ++i) {
                snprintf(buffer, 50, "meccanica_sx_%d", i);
                meccaniche_sx[i] = create_copy(meccaniche_sx[0], buffer);
                set_translate(meccaniche_sx[i], delta * i, delta2 * i,
                              -delta3 * i);
                out_x[i] = out_x[0] + delta * i;
                out_y[i] = out_y[0] + delta2 * i;
        }
        /* ...e a destra */
        delta2 = 0.5 * tan(16.0 * M_PI / 180.0);

        attacco_corda = read_nurbs("attacco_corda_dx.db");
        attacco_sotto = read_nurbs("attacco_corda_sotto_dx.db");
        attacco_vite = read_nurbs("attacco_vite_dx.db");
        vite = read_nurbs("vite_dx.db");

        meccaniche_dx[0] = create_list(attacco_corda, "meccanica_dx_0");
        meccaniche_dx[0] = insert_in_list(meccaniche_dx[0], attacco_sotto);
        meccaniche_dx[0] = insert_in_list(meccaniche_dx[0], attacco_vite);
        meccaniche_dx[0] = insert_in_list(meccaniche_dx[0], vite);

        out_x[5] = 8.592;
        out_y[5] = -0.4;

        for (i = 1; i < 3; ++i) {
                snprintf(buffer, 50, "meccanica_dx_%d", i);
                meccaniche_dx[i] = create_copy(meccaniche_dx[0], buffer);
                set_translate(meccaniche_dx[i], delta * i, delta2 * i,
                              -delta3 * i);
                out_x[5-i] = out_x[5] + delta * i;
                out_y[5-i] = out_y[5] + delta2 * i;
        }

        meccaniche = create_list(meccaniche_sx[0], "meccaniche");
        meccaniche = insert_in_list(meccaniche, meccaniche_dx[0]);
        for (i = 1; i < 3; ++i) {
                meccaniche = insert_in_list(meccaniche, meccaniche_sx[i]);
                meccaniche = insert_in_list(meccaniche, meccaniche_dx[i]);
        }

        return meccaniche;
}


ITEM costruisci_porta_chitarra(void)
{
        ITEM tmp;

        ITEM porta_chitarra, corpo, base, appoggio_manico, tappi;

        ITEM gomma_nera;

        gomma_nera = create_attribute("gomma_nera");
        set_color(gomma_nera, 0, 0, 0, 0, 0);
        set_reflectivity(gomma_nera, 0, 10);

        tmp = read_nurbs("corpo.db");
        corpo = create_list(tmp, "corpo");
        tmp = read_nurbs("piede1.db");
        corpo = insert_in_list(corpo, tmp);
        tmp = read_nurbs("piede2.db");
        corpo = insert_in_list(corpo, tmp);
        tmp = read_nurbs("piede3.db");
        corpo = insert_in_list(corpo, tmp);

        set_attribute(corpo, bianco);

        tmp = read_nurbs("appoggio_manico.db");
        appoggio_manico = create_list(tmp, "appoggio_manico");
        tmp = read_nurbs("appoggio_manico_tappo1.db");
        appoggio_manico = insert_in_list(appoggio_manico, tmp);
        tmp = read_nurbs("appoggio_manico_tappo2.db");
        appoggio_manico = insert_in_list(appoggio_manico, tmp);

        set_attribute(appoggio_manico, gomma_nera);

        tmp = read_nurbs("base.db");
        base = create_list(tmp, "base");
        tmp = read_nurbs("base_fine1.db");
        base = insert_in_list(base, tmp);
        tmp = read_nurbs("base_fine2.db");
        base = insert_in_list(base, tmp);
        tmp = read_nurbs("base_fine_tappo1.db");
        base = insert_in_list(base, tmp);
        tmp = read_nurbs("base_fine_tappo2.db");
        base = insert_in_list(base, tmp);

        set_attribute(base, gomma_nera);

        tmp = read_nurbs("tappo1.db");
        tappi = create_list(tmp, "tappi");
        tmp = read_nurbs("tappo2.db");
        tappi = insert_in_list(tappi, tmp);
        tmp = read_nurbs("tappo3.db");
        tappi = insert_in_list(tappi, tmp);

        set_attribute(tappi, gomma_nera);

        porta_chitarra = create_list(corpo, "porta_chitarra");
        porta_chitarra = insert_in_list(porta_chitarra, appoggio_manico);
        porta_chitarra = insert_in_list(porta_chitarra, base);
        porta_chitarra = insert_in_list(porta_chitarra, tappi);

        return porta_chitarra;
}


ITEM costruisci_stanza(void)
{
        ITEM stanza;
        ITEM pavimento, parete_fondo, parete_sx;
        ITEM battiscopa_fondo, battiscopa_sx;
        ITEM parete_specchio;

        ITEM cera, specchio;

        cera = create_attribute("cera");
        set_color(cera, 0, 0, 0.2, 0.6, 0.6);
        set_reflectivity(cera, 0.05, 4);

        specchio = create_attribute("specchio");
        set_color(specchio, 0, 0, 0, 0, 1);
        set_reflectivity(specchio, 1, 2);

        pavimento = read_nurbs("pavimento.db");
        set_scale(pavimento, 0.1, 0.1, 1);
        set_projection_texture(pavimento, cera,"textures/parque.hr",
                               2, 1, 0.3, 0.3);
        set_scale(pavimento, 10, 10, 1);
        set_translate(pavimento, 0, 0, -9);

        parete_fondo = read_nurbs("parete_fondo.db");

        parete_specchio = create_copy(parete_fondo, "parete_specchio");
        set_attribute(parete_specchio, specchio);

        set_scale(parete_fondo, 0.25 / 0.667, 0.25, 0.25);
        set_projection_texture(parete_fondo, bianco,"textures/carta_parati.hr", 1, 1, 0.3, 0.3);
        set_scale(parete_fondo, 4 * 0.667, 4, 4);

        parete_sx = create_copy(parete_fondo, "parete_sx");

        set_z_rotate(parete_sx, 90, 1);

        set_translate(parete_fondo, 1.25, 11.5, 6);
        set_translate(parete_sx, -11.5, -1.25, 6);
        set_translate(parete_specchio, 0, -30, 6);

        battiscopa_fondo = read_nurbs("battiscopa_fondo.db");
        set_scale(battiscopa_fondo, 0.25, 1, 0.3);
        set_translate(battiscopa_fondo, 0, 0, 0.3);
        set_projection_texture(battiscopa_fondo, bianco,"textures/battiscopa.hr", 1, 1, 0.3, 0.3);
        set_translate(battiscopa_fondo, 0, 0, -0.3);
        set_scale(battiscopa_fondo, 4, 1, 1.0 / 0.3);

        battiscopa_sx = create_copy(battiscopa_fondo, "battiscopa_sx");

        set_z_rotate(battiscopa_sx, 90, 1);

        set_translate(battiscopa_fondo, 0, 11.5, -9);
        set_translate(battiscopa_sx, -11.5, 0, -9);

        stanza = create_list(pavimento, "stanza");
        stanza = insert_in_list(stanza, parete_specchio);
        stanza = insert_in_list(stanza, parete_fondo);
        stanza = insert_in_list(stanza, battiscopa_fondo);
        stanza = insert_in_list(stanza, parete_sx);
        stanza = insert_in_list(stanza, battiscopa_sx);

        return stanza;
}
